from datetime import datetime

import click


def color_echo(text, color, **kwargs):
    if color or kwargs:
        text = click.style(text, fg=color, **kwargs)
    click.echo(text)


def today() -> datetime:
    return datetime.now()



import os
import subprocess
import sys

from src.helper.color_path import Path

venv_path = Path.cwd() / "venv"


def run_in_venv(cmd: str, check=True):
    env = os.environ.copy()
    env["VIRTUAL_ENV"] = str(venv_path)
    env["PATH"] = str(Path.cwd()) + env["PATH"]

    subprocess.run(cmd, env=env, check=check, shell=True)


def create_venv(venv_dir=".venv"):
    # TODO: this is temp code
    venv_dir = Path(venv_dir)
    # Create the virtual environment
    if not os.path.exists(venv_dir):
        subprocess.run([sys.executable, "-m", "venv", venv_dir.name], check=True)
        print(f"Virtual environment created at {venv_dir}")
    else:
        print(f"Virtual environment already exists at {venv_dir}")

    if venv_dir.joinpath("Scripts").exists():
        subprocess.run([f"{venv_dir.name}/Scripts/activate"], check=True)
    else:
        subprocess.run([f"source {venv_dir.name}/bin/activate"], check=True)


def install_requirements(venv_dir="venv", requirements_file="requirements.txt"):
    # TODO: this is temp code
    # Path to pip in the virtual environment
    pip_executable = (
        os.path.join(venv_dir, "Scripts", "pip") if os.name == "nt" else os.path.join(venv_dir, "bin", "pip")
    )

    if os.path.exists(requirements_file):
        subprocess.run([pip_executable, "install", "-r", requirements_file], check=True)
        print(f"Installed requirements from {requirements_file}")
    else:
        print(f"Requirements file {requirements_file} not found.")


if __name__ == "__main__":
    VENV_DIRECTORY = ".venv"  # Change as needed
    REQ_FILE = "requirements.txt"  # Change as needed

    create_venv(VENV_DIRECTORY)
    install_requirements(VENV_DIRECTORY, REQ_FILE)



# pylint: disable =invalid-name
from enum import Enum

try:
    from enum import StrEnum
except ImportError:
    # py3.10
    class StrEnum(str, Enum):
        def __str__(self):
            return str(self.value)


class Facet(StrEnum):
    contribution = "contribution"
    git = "git"
    gitignore = ".gitignore"
    license = "license"
    pre_commit = "pre-commit"
    pyproject = "pyproject"
    readme = "readme"
    sourcery = "sourcery"


class RootFile(StrEnum):
    contribution = "CONTRIBUTING.md"
    gitignore = ".gitignore"
    license = "LICENSE"
    pre_commit_yaml = ".pre-commit-config.yaml"
    pylint = ".pylintrc"
    pyproject_toml = "pyproject.toml"
    readme = "README.md"
    sourcery_config = ".sourcery.yaml"


class Template(StrEnum):
    gitignore = "gitignore.txt"
    pre_commit_pref = "pre_commit.toml"
    tests = "tests.toml"


class Workflow(StrEnum):
    pre_commit = "code-lint.yml"
    pytest = "code-test.yml"

    def path(self):
        return f"./.github/workflows/{self.value}"



import pytest

from src.helper.color_path import COLOR_MAP, Path


def test_colored_path():
    test_path = Path("fol/file.py")
    assert not test_path.exists()
    assert Path.cwd()
    for _format in COLOR_MAP:
        assert f"{test_path:{_format}}"

    with pytest.raises(KeyError):
        assert f"{test_path:ko}"



import subprocess


def get_git_user() -> str:
    try:
        result = subprocess.run(["git", "config", "--get", "user.name"], capture_output=True, text=True, check=True)
    except subprocess.CalledProcessError:
        return ""
    return result.stdout.strip()



import os
from unittest.mock import Mock, patch

import pytest
from click.testing import CliRunner


@pytest.fixture(scope="session")
def runner():
    return CliRunner()


@pytest.fixture(scope="session")
def test_env(tmp_path_factory):
    """
    Creates a complete test environment with predefined structure

    path = %temp%/pytest-of-App
    """
    tmp_dir = tmp_path_factory.getbasetemp()
    (tmp_dir / "is_tmp").touch()  # mandatory for debugging

    # Add sample data
    sample_req = "\n".join(["yuhi", "panda", "type", "isort"])
    (tmp_dir / "requirements.txt").write_text(sample_req)
    (tmp_dir / "main.py").write_text(get_test_code())

    original_cwd = os.getcwd()
    os.chdir(tmp_dir)
    try:
        yield tmp_dir
    finally:
        # Change back to the original directory
        os.chdir(original_cwd)


def get_test_code() -> str:
    with open("tests/test_files/code.txt", "r", encoding="U8") as f:
        return f.read()


@pytest.fixture
def mock_httpx_get():
    with patch("httpx.get") as mock_get:
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.raise_for_status.return_value = None
        mock_get.return_value = mock_response
        yield mock_get


# no tests run here



# pylint: disable=W0613
import datetime as dt
from unittest.mock import AsyncMock, patch

import httpx

from src.cli import cli


def test_sample(runner, test_env):
    response = runner.invoke(cli, ["sample", "files"])
    if response.exit_code == 0:
        assert "project.yaml" in response.output
        assert "created" in response.output

        response = runner.invoke(cli, ["sample", "files"])
        assert response.exit_code == 0
        assert "SKIPPING" in response.output
    else:
        assert "NotImplementedError" in response.output


@patch("httpx.AsyncClient.get", new_callable=AsyncMock)
def test_scan(mock_async_fn, runner, test_env):
    now = dt.datetime.now().isoformat(timespec="seconds")
    old = dt.datetime(2020, 1, 1).isoformat(timespec="seconds")
    mock_data = {"yuhi": (200, now, now), "panda": (200, old, old), "type": (404, now, now), "isort": (200, old, now)}

    def mock_response(url: str):
        pkg = url.rsplit("/", maxsplit=2)[-2]
        status, first_upload, last_upload = mock_data[pkg]
        return httpx.Response(
            status_code=status,
            json={"releases": {"0.0.1": [{"upload_time": first_upload}]}, "urls": [{"upload_time": last_upload}]},
        )

    mock_async_fn.side_effect = mock_response
    response = runner.invoke(cli, ["scan"])
    assert response.exit_code == 0
    assert "3 bad package(s)" in response.output



# pylint: disable=w0613
from unittest.mock import patch

import pytest

from src.cli import cli


def assert_with_error(result, msg):
    if result.exit_code == 0:
        assert msg in result.output
    else:
        assert "NotImplementedError" in result.output


@pytest.mark.parametrize("facet", ["contribution", ".gitignore", "readme", "sourcery"])
def test_add_facet(test_env, runner, facet):
    """Test adding various facets."""
    result = runner.invoke(cli, ["add", facet])
    print(result.output)
    assert_with_error(result, "created")


def test_add_license(test_env, runner):
    gh_service = "src.facets.repository"
    with (
        patch("questionary.select") as mock_select,
        patch(f"{gh_service}.get_licenses", return_value={"MIT License": "mit"}),
        patch(f"{gh_service}.get_license_content", return_value="[year][fullname]<program>"),
    ):
        mock_select.return_value.ask.return_value = "MIT License"

        result = runner.invoke(cli, ["add", "license"])
        assert_with_error(result, "created")


def test_add_pre_commit(test_env, runner):
    result = runner.invoke(cli, ["add", "pre-commit"])
    assert_with_error(result, "pre-commit run --all-files")


def test_add_pylint(test_env, runner):
    result = runner.invoke(cli, ["add", "pylint"])
    assert_with_error(result, ".pylintrc:1 : created")



import click

from src.common import append_file, create_file, get_template, get_workflow
from src.helper import color_echo, read_toml, read_yaml, yaml
from src.helper.color_path import Path
from src.helper.constants import RootFile, Template, Workflow

from .pylint_handler import get_pylint_config


def _add_hooks_yaml(*hooks: str):
    def add_space_to_bracket(x):
        return x.replace("[", "[ ").replace("]", " ]")

    # Get current and template configs
    pre_commit_yaml = Path(RootFile.pre_commit_yaml)
    if not pre_commit_yaml.exists():
        pre_commit_yaml.write_text("repos: []\n", encoding="U8")
    current_config = read_yaml(RootFile.pre_commit_yaml)

    with get_template(RootFile.pre_commit_yaml) as f:
        template_config = yaml.load(f)
    hooks = hooks or template_config["default_hooks"]

    # check hook id before merge
    _current_hooks = set()
    for repo in current_config["repos"]:
        if repo.get("repo", "").endswith("pre-commit/pre-commit-hooks"):
            _current_hooks.add("pre-commit-hooks")
            continue
        if "hooks" not in repo:
            continue
        for hook in repo["hooks"]:
            _current_hooks.add(hook["id"])
    for hook in hooks:
        if hook in _current_hooks:
            continue
        current_config["repos"].append(template_config[hook])
        click.echo(f"\tADDED : {hook}")

    print("writing to pre-commit-yaml file")
    with open(RootFile.pre_commit_yaml, "w", encoding="U8") as f:
        yaml.dump(current_config, f, transform=add_space_to_bracket)


def _add_hooks_toml(*hooks: str):
    """Adds [tool.hook] configs in toml file"""
    with get_template(Template.pre_commit_pref) as f:
        config = f.read()
        black, flake8, isort = config.split("\n\n")
    config_map = {"black": black, "flake8": flake8, "isort": isort}

    pyproject = Path(RootFile.pyproject_toml)
    if not pyproject.exists():
        pyproject.write_text("", encoding="U8")

    current_config = read_toml(RootFile.pyproject_toml)

    config = ""
    for hook in hooks:
        if hook not in current_config.get("tool", {}):
            config += f"\n\n{config_map[hook]}"
        else:
            click.echo(f"\tSKIPPED : {hook} is already configured")
    if config:
        append_file(RootFile.pyproject_toml, config)


@click.command("pre-commit")
def add_pre_commit():
    """Integrates pre-commit tool"""
    click.echo("setting up pre-commit..")
    _add_hooks_yaml()
    _add_hooks_toml("black", "flake8", "isort")

    click.echo("adding pre-commit workflow...")
    with get_workflow(Workflow.pre_commit) as f:
        config = f.read()
    create_file(Workflow.pre_commit.path(), config)

    # TODO: run installations and autoupdates, after .venv setup
    click.echo("-- pre-commit files created. --\nNow run below commands:")
    color_echo("\tpre-commit install && pre-commit autoupdate", "cyan")
    color_echo("\tpre-commit run --all-files", "cyan")


@click.command("pylint")
def add_pylint():
    """Adds pylint facet"""
    click.echo("adding pylint...")
    create_file(RootFile.pylint, get_pylint_config(), use_file_prefix=True)

    _add_hooks_yaml("pylint")


@click.command("sourcery")
def add_sourcery():
    """Adds sourcery facet"""
    click.echo("adding sourcery...")
    with get_template(RootFile.sourcery_config) as f:
        config = f.read()
    create_file(RootFile.sourcery_config, config)



import click

from src.helper import Path, yaml


@click.command("pyproject")
def add_pyproject():
    """Adds pyproject.toml file"""
    click.echo("NOT IMPLEMENTEDŪ")
    # pyproject = read_toml(RootFile.pyproject_toml)
    # print(pyproject["tool"]["black"])


def _create_files(structure_data: str | list | dict, parent_path=Path()):
    """Recursively traverses a nested YAML structure and create files"""
    if isinstance(structure_data, dict):
        for key, value in structure_data.items():
            _create_files(value, parent_path / key)
    elif isinstance(structure_data, list):
        for item in structure_data:
            _create_files(item, parent_path)
    else:
        # Makes parent folders and blank file
        file = parent_path / structure_data
        parent_path.mkdir(parents=True, exist_ok=True)
        try:
            file.touch(exist_ok=False)
            click.echo(f"CREATED : {file!s}")
        except FileExistsError:
            click.echo(f"SKIPPED : {file!s}")


@click.command("files")
@click.option("--structure-file", "-s", default="project.yaml", required=True)
def create_structure(structure_file: str):
    """Creates all folders and files as structured in project.yaml"""
    filename = Path(structure_file)
    assert filename.suffix in {".yml", ".yaml"}, "Only YAML file is supported"
    with open(filename, "rb") as f:
        data = yaml.load(f)

    try:
        project_name = data["project"]["name"]
    except KeyError:
        click.echo("project.name is missing", err=True, color=True)
        return
    if project_name not in data:
        click.echo(f"No project structure available for '{project_name}'", err=True)
        return

    _create_files(structure_data=data[project_name], parent_path=Path(project_name))



from pathlib import Path

import click

COLOR_MAP = {"new": "green", "+": "green", "skip": "bright_green", "old": "yellow", "del": "red", "-": "red"}


def formatter(self, format_spec):
    _path = str(self)
    return click.style(_path, fg=COLOR_MAP[format_spec.lower()]) if format_spec else _path


Path.__format__ = formatter



# pragma: no cover
import httpx

GITHUB_API = "https://api.github.com"


def get_licenses() -> dict:
    """Fetches all available licenses from Github"""
    response = httpx.get(f"{GITHUB_API}/licenses", timeout=5.0)
    response.raise_for_status()
    return {x["name"]: x["key"] for x in response.json()}


def get_license_content(key) -> str | None:
    resp = httpx.get(f"{GITHUB_API}/licenses/{key}", timeout=90)
    resp.raise_for_status()
    return resp.json()["body"]



import asyncio
from dataclasses import dataclass
from datetime import datetime
from typing import Awaitable, Callable, Iterable

import click
import httpx

today = datetime.now()
MIN_ACTIVE_MONTHS = 10
MAX_INACTIVE_MONTHS = 60


@dataclass
class PackageData:
    name: str
    status_code: int
    first_upload_date: datetime
    last_upload_date: datetime


ScannerFn = Callable[..., Awaitable[PackageData]]


async def scan_pypi(client: httpx.AsyncClient, package: str) -> PackageData:
    def to_date(date_str: str) -> datetime:
        if not date_str:
            return datetime.now()
        # Invalid isoformat issue in CI, hence using format string
        return datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%S")

    response = await client.get(f"https://pypi.org/pypi/{package}/json")

    first_upload_date = last_upload_date = None
    _upload_time_field = "upload_time"

    if response.status_code == 200:
        pkg_data = response.json()
        releases = pkg_data["releases"]
        for _, _data in releases.items():
            if _data:
                first_upload_date = _data[0][_upload_time_field]
                break

        last_upload_date = pkg_data["urls"][0][_upload_time_field]

    return PackageData(package, response.status_code, to_date(first_upload_date), to_date(last_upload_date))


def get_dependencies() -> Iterable:
    packages = set()
    with open("requirements.txt", "r", encoding="U8") as file:
        packages.update(file.read().splitlines())
    return packages


def get_package_names(pkg: str) -> str:
    return pkg.split("=", 1)[0].rstrip().rstrip(">~")


def _age_in_months(dt: datetime) -> int:
    return (today.year - dt.year) * 12 + (today.month - dt.month)


async def scan_all_packages(scanner: ScannerFn, packages: list[str]) -> int:
    bad_count = 0
    limits = httpx.Limits(max_connections=10, max_keepalive_connections=5)

    async with httpx.AsyncClient(limits=limits, timeout=20.0) as client:
        tasks = [scanner(client, package) for package in packages]
        results: tuple[PackageData] = await asyncio.gather(*tasks, return_exceptions=True)

    for package_data in results:
        if isinstance(package_data, Exception):
            click.echo(f"{package_data}: report a bug")
            continue
        if package_data.status_code == 404:
            bad_count += 1
            click.echo(f"{package_data.name}: INVALID / DELISTED")
        elif package_data.status_code == 200:
            if _age_in_months(package_data.last_upload_date) > MAX_INACTIVE_MONTHS:
                bad_count += 1
                click.echo(f"{package_data.name}: OUTDATED")
            elif _age_in_months(package_data.first_upload_date) < MIN_ACTIVE_MONTHS:
                bad_count += 1
                click.echo(f"{package_data.name}: RISKY")
    return bad_count


def run_scan(package_repo: str = "pypi"):
    package_repo = package_repo.lower()
    if package_repo == "pypi":
        _scanner = scan_pypi
    else:
        click.echo("Unsupported package repository")
        return

    packages = [get_package_names(x) for x in get_dependencies()]
    click.echo(f"Scanning {len(packages)} packages ...")
    bad_pkg_count = asyncio.run(scan_all_packages(_scanner, packages))

    click.echo(f"{bad_pkg_count} bad package(s) found")


if __name__ == "__main__":
    asyncio.run(scan_all_packages(scan_pypi, ["panda", "isort", "yuhi", "type"]))



"""first tests"""

import pytest

from src.cli import cli


def test_yuhi_help(runner):
    result = runner.invoke(cli, ["--help"])
    assert result.exit_code == 0


@pytest.mark.parametrize("command", ["add", "create", "sample", "scan"])
def test_command_help(runner, command):
    result = runner.invoke(cli, [command, "--help"])
    assert result.exit_code == 0


@pytest.mark.parametrize(
    "facet", ["contribution", ".gitignore", "license", "pre-commit", "pylint", "pyproject", "readme", "sourcery"]
)
def test_add_facet_help(runner, facet):
    result = runner.invoke(cli, ["add", facet, "--help"])
    assert result.exit_code == 0



# sphinx config for documentation
# pylint: disable=invalid-name

project = "Yuhi (यूँ ही)"
description = "Python CLI toolkit to automate project enhancement using standard config templates."
authors = "Rishabh B"
version = "0.1.0"
project_copyright = "%Y, Rishabh B"

extensions = [
    "sphinx.ext.duration",
    "sphinx.ext.doctest",
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.intersphinx",
]

intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master/", None),
}

templates_path = ["_templates"]
exclude_patterns = []

html_theme = "alabaster"

html_theme_options = {
    "github_user": "rsb-23",
    "github_repo": "yuhi",
    "github_type": "star",
    "github_button": "true",
    "github_count": "true",
}

html_static_path = ["_static"]



"""Facets related to git repositories"""

import re

import click
import questionary

from src.common import create_file, get_template
from src.helper import Path, today
from src.helper.constants import RootFile, Template
from src.services import get_git_user, get_license_content, get_licenses


@click.command("contribution")
def add_contribution():
    """Adds a CONTRIBUTION file to the repo"""
    click.echo("Adding CONTRIBUTION file..")
    create_file(RootFile.contribution)


# @click.command("gitignore")
@click.command(".gitignore")
def add_gitignore():
    """Adds a .gitignore file to git repo"""
    click.echo("Adding .gitignore file..")
    with get_template(Template.gitignore) as f:
        config = f.read()
    create_file(RootFile.gitignore, config)


@click.command("license")
def add_license():
    """Adds a license based on github template"""

    filepath = Path(RootFile.license)
    if filepath.exists():
        click.echo(f"SKIPPING : {filepath:skip} already exists")
        return

    licenses = get_licenses()
    license_choices = list(licenses.keys())

    name = questionary.select("Select a license", choices=license_choices).ask()
    click.echo(f"Adding {name} license..")

    year, author = str(today().year), get_git_user() or "<AUTHOR>"
    content: str = get_license_content(key=licenses[name])
    tags = re.findall(r"[\[<][\w\s]+[>\]]", content)
    print(tags)
    for tag in tags:
        if tag[1:5] in ("year", "yyyy"):
            new_val = year
        elif "name" in tag:
            new_val = author
        else:
            new_val = tag
        content = content.replace(tag, new_val)

    if create_file(RootFile.license, content):
        click.echo(f"LICENSE ADDED: {name}", color=True)


@click.command("readme")
@click.option("--ext", default="md")
def add_readme(ext):
    """Adds a README file to the repo"""
    click.echo(f"Adding readme with {ext}..")
    filename = RootFile.readme
    if ext != "md":
        filename = f"{filename[:-2]}{ext}"
    content = b"# Project Name\n---\nDescription"
    create_file(filename, content)



from .code_quality import add_pre_commit, add_pylint, add_sourcery
from .project import add_pyproject, create_structure
from .repository import add_contribution, add_gitignore, add_license, add_readme

SUBGROUP_ADD = [
    add_contribution,
    add_gitignore,
    add_license,
    add_pre_commit,
    add_pylint,
    add_pyproject,
    add_readme,
    add_sourcery,
]
SUBGROUP_CREATE = [create_structure]



import click

from src.__about__ import __version__
from src.commands import add, create, sample, scan
from src.glossary import Glossary


@click.group(cls=Glossary)
@click.version_option(__version__, "-V", "--version")
@click.help_option("-h", "--help")
def cli():
    """Yuhi : A casual project setup tool"""


cli.add_command(add)  # noqa
cli.add_command(create)  # noqa
cli.add_command(sample)  # noqa
cli.add_command(scan)  # noqa

if __name__ == "__main__":
    cli()



from contextlib import contextmanager
from copy import deepcopy
from functools import partial
from importlib.resources import files

import click

from src.helper.color_path import Path

FILE_PREFIX = "yuhi-"


@contextmanager
def get_template(filename: str, folder: str = "templates"):
    # Access file content
    resource = files(folder).joinpath(filename)
    with resource.open("r") as file:
        yield file


get_sample = partial(get_template, folder="samples")
get_workflow = partial(get_template, folder="templates.workflow")


def create_file(filepath: str, content: str | bytes = b"", use_file_prefix=False) -> bool:
    filepath = Path(filepath)
    if filepath.exists():
        click.echo(f"SKIPPING : {filepath:skip} already exists")
        if not use_file_prefix:
            return False
        filepath = Path(FILE_PREFIX + filepath.name)
        click.echo(f"USING : {filepath:new} instead")

    if isinstance(content, str):
        content = content.encode()
    filepath.parent.mkdir(parents=True, exist_ok=True)
    filepath.write_bytes(content)
    click.echo(f"{filepath:new}:1 : created")
    return True


def append_file(filepath: str, content: str | bytes):
    filepath = Path(filepath)
    if isinstance(content, str):
        content = content.encode()
    with open(filepath, "r", encoding="U8") as fw:
        count = len(fw.readlines())
    with open(filepath, "ab") as fw:
        fw.write(content)
    click.echo(f"{filepath:old}:{count + 1} : content appended")


def deep_merge(dict1: dict, dict2: dict) -> dict:
    result = deepcopy(dict1)

    for key, value in dict2.items():
        if key in result and isinstance(result[key], dict) and isinstance(value, dict):
            # Recursively merge nested dictionaries
            result[key] = deep_merge(result[key], value)
        else:
            # Override or add values from dict2
            result[key] = deepcopy(value)

    return result



from ruamel.yaml import YAML

try:
    import tomllib as toml
except ImportError:
    # py3.10
    import toml  # noqa

yaml = YAML(typ="rt")
yaml.indent(mapping=2, sequence=4, offset=2)
yaml.preserve_quotes = True
yaml.width = 100
yaml.line_break = True


def read_toml(filename: str) -> dict:
    try:
        with open(filename, "rb") as f:
            content = toml.load(f)
    except TypeError:
        with open(filename, "r", encoding="U8") as f:
            content = toml.load(f)
    return content


def read_yaml(filename: str) -> dict:
    with open(filename, "rb") as f:
        content = yaml.load(f)
    return content



from collections import defaultdict
from operator import itemgetter

import click

from src.helper.color_path import Path

from .pkg_install import run_in_venv

PYLINT_DEFAULT_DISABLES = {"missing-docstring": "C", "duplicate-code": "R"}
NAME_ARG_MAP = {
    "too-many-lines": "max-module-lines",
    "too-many-arguments": "max-args",
    "too-many-boolean-expressions": "max-bool-expr",
    "too-many-instance-attributes": "max-attributes",
}
PYLINT_REPORT = Path("yuhi-pylint.txt")


def get_lint_errors():
    with open(PYLINT_REPORT, "r", encoding="U8") as f:
        lint_errors = f.readlines()
    yield from lint_errors


def _generate_report() -> (dict, dict):
    fetcher = itemgetter(1, -2, -1)

    unique_messages = {x: defaultdict(int) for x in "FEWCR"}
    max_config = {}
    for line in get_lint_errors():
        if not line:
            continue

        code, outbound, name = fetcher(line.split())
        name = name[1:-1]
        if "too-many" in name:
            max_config[name] = max(int(outbound.split("/")[0][1:]), max_config.get(name, 0))
        elif code[0] in "FEWCR":
            unique_messages[code[0]][name] += 1

    for err_name, code0 in PYLINT_DEFAULT_DISABLES.items():
        unique_messages[code0][err_name] = 0
    return unique_messages, max_config


def get_config_arg(check_name):
    return NAME_ARG_MAP.get(check_name, f"max-{check_name[9:]}")


def _run_pylint(folders=None):
    if not folders:
        folders = ["src", "test", "tests"]
    disables = ",".join(["I", "F0001", *PYLINT_DEFAULT_DISABLES.keys()])
    sources = " ".join(f"./{fol}" for fol in folders if Path(fol).exists())

    click.echo(f"Running pylint on: ./*.py {sources}")
    run_in_venv(f"pylint ./*.py {sources} -sn --enable=all --disable={disables} > {PYLINT_REPORT}", check=False)
    click.echo("Report generated")


def get_pylint_config() -> str:
    def get_config_lines() -> str:
        config_lines = [f"{get_config_arg(name)} = {limit}" for name, limit in max_config.items()]
        return "\n".join(sorted(config_lines)) or ""

    _run_pylint()
    unique_messages, max_config = _generate_report()
    disable_errors = "disable=\n\tI,"
    error_sep = "\n\t"
    for k, v in unique_messages.items():
        if v:
            msgs_with_count = (f"{msg}, #{count or 'keep'}" for msg, count in sorted(v.items()))
            disable_errors += f"\n\t# {k}\n\t{error_sep.join(msgs_with_count)}"
    pylint_config = f"""[MAIN]\njobs=2\nignore-patterns=^(?!^(src|tests?|.+.py)$).+$\n
[FORMAT]\nmax-line-length = 130\n
[DESIGN]\n{get_config_lines()}\n
[MESSAGES CONTROL]\n{disable_errors}
"""
    return pylint_config.strip()



import click

from src.common import create_file, get_sample
from src.facets import SUBGROUP_ADD, SUBGROUP_CREATE
from src.facets.scanner import run_scan


@click.group()
def add():
    """Adds a facet to the project"""


for cmd in SUBGROUP_ADD:
    add.add_command(cmd)


@click.group()
def create():
    """Creates complete project structure"""


for cmd in SUBGROUP_CREATE:
    create.add_command(cmd)


@click.command()
@click.argument("name", type=click.Choice(["files", "project"]))
def sample(name):
    """Adds a sample yaml file to local"""
    if name == "files":
        sample_name = "project.yaml"
    else:
        sample_name = f"{name}.yaml"
    with get_sample(sample_name) as f:
        create_file(sample_name, content=f.read())


@click.command()
def scan():
    """lists all bad packages in requirements.txt"""
    run_scan()



import click

DICTIONARY = {"facet": "A sub-component of a project"}


# pylint: disable=r0903
class Glossary(click.Group):
    """Dictionary of terms used in the help"""

    def format_help(self, ctx, formatter):
        super().format_help(ctx, formatter)

        with formatter.section("Glossary"):
            formatter.write_dl([*DICTIONARY.items()])
